@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.2
+ *  CU sudo version 1.5.4
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
  *
- *  $Id: compat.h,v 1.27 1996/10/09 17:37:18 millert Exp $
+ *  $Id: compat.h,v 1.30 1998/01/13 04:48:01 millert Exp $
  */
 
 #ifndef _SUDO_COMPAT_H
@@ -138,7 +138,7 @@
 #  ifdef __hpux
 #    define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID, UID_NO_CHANGE))
 #  else
-#    define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID))
+#    define seteuid(_EUID)	(setreuid(UID_NO_CHANGE, _EUID))
 #  endif /* __hpux */
 #endif /* HAVE_SETEUID */
 
