@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.2
+ *  CU sudo version 1.5.4
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
  *
- *  $Id: options.h,v 1.32 1996/10/07 05:05:33 millert Exp $
+ *  $Id: options.h,v 1.35 1998/01/13 04:48:42 millert Exp $
  */
 
 #ifndef _SUDO_OPTIONS_H
@@ -55,7 +55,7 @@
 #define MAILER _PATH_SENDMAIL	/* what mailer to use */
 #define UMASK 0022		/* umask that the root-run prog should use */
 #define INCORRECT_PASSWORD "Sorry, try again." /* message for bad passwd */
-#define MAILSUBJECT "*** SECURITY information ***" /* subject of mail sent */
+#define MAILSUBJECT "*** SECURITY information for %h ***" /* mail subject */
 #define PASSPROMPT "Password:"	/* default password prompt */
 /*#define IGNORE_DOT_PATH	/* ignore '.' in $PATH if it exists */
 /*#define SECURE_PATH	"/bin:/usr/ucb:/usr/bin:/usr/etc:/etc" /* secure path */
