@@ -1,9 +1,9 @@
 /*
- * CU sudo version 1.4 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5.4 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
- * Please send bugs, changes, problems to sudo-bugs@cs.colorado.edu
+ * Please send bugs, changes, problems to sudo-bugs@courtesan.com
  *
  */
 
@@ -47,11 +47,11 @@
  *   However, due to the fact that both of the above are no longer
  *   working at Root Group, I am maintaining the "CU version" of
  *   sudo.
- *		Todd Miller  <Todd.Miller@cs.colorado.edu>
+ *		Todd Miller  <Todd.Miller@courtesan.com>
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: sudo.c,v 1.143 1996/03/22 00:55:18 millert Exp $";
+static char rcsid[] = "$Id: sudo.c,v 1.179 1998/01/13 04:48:42 millert Exp $";
 #endif /* lint */
 
 #define MAIN
@@ -83,10 +83,14 @@ static char rcsid[] = "$Id: sudo.c,v 1.143 1996/03/22 00:55:18 millert Exp $";
 #include <sys/param.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#if defined(__osf__) && defined(HAVE_C2_SECURITY)
-#include <sys/security.h>
-#include <prot.h>
-#endif /* __osf__ && HAVE_C2_SECURITY */
+#if (SHADOW_TYPE == SPW_SECUREWARE)
+#  ifdef __hpux
+#    include <hpsecurity.h>
+#  else
+#    include <sys/security.h>
+#  endif /* __hpux */
+#  include <prot.h>
+#endif /* SPW_SECUREWARE */
 #ifdef HAVE_DCE
 #include <pthread.h>
 #endif /* HAVE_DCE */
@@ -123,10 +127,11 @@ static void usage			__P((int));
 static void load_globals		__P((int));
 static int check_sudoers		__P((void));
 static void load_cmnd			__P((int));
-static void add_env			__P((void));
+static void add_env			__P((int));
 static void clean_env			__P((char **, struct env_table *));
-extern int user_is_exempt		__P((void));
+extern int  user_is_exempt		__P((void));
 extern struct passwd *sudo_getpwuid	__P((uid_t));
+extern void list_matches		__P((void));
 
 /*
  * Globals
@@ -136,13 +141,16 @@ char **Argv;
 int NewArgc = 0;
 char **NewArgv = NULL;
 struct passwd *user_pw_ent;
+char *runas_user = "root";
 char *cmnd = NULL;
-char *tty = NULL;
-char *prompt = PASSPROMPT;
+char *cmnd_args = NULL;
+char *tty = "unknown";
+char *prompt;
 char host[MAXHOSTNAMELEN + 1];
 char *shost;
 char cwd[MAXPATHLEN + 1];
 struct stat cmnd_st;
+static char *runas_homedir = NULL;
 extern struct interface *interfaces;
 extern int num_interfaces;
 extern int printmatches;
@@ -163,6 +171,8 @@ struct env_table badenv_table[] = {
 #ifdef HAVE_KERB4
     { "KRB_CONF", 8 },
 #endif
+    { "ENV=", 4 },
+    { "BASH_ENV=", 9 },
     { (char *) NULL, 0 }
 };
 
@@ -182,9 +192,9 @@ int main(argc, argv)
     int sudo_mode = MODE_RUN;
     extern char ** environ;
 
-#if defined(SHADOW_TYPE) && (SHADOW_TYPE == SPW_SECUREWARE)
+#if (SHADOW_TYPE == SPW_SECUREWARE)
     (void) set_auth_parameters(argc, argv);
-#endif /* SHADOW_TYPE && SPW_SECUREWARE */
+#endif /* SPW_SECUREWARE */
 
     Argv = argv;
     Argc = argc;
@@ -194,27 +204,33 @@ int main(argc, argv)
 	exit(1);
     }
 
+    /*
+     * set the prompt based on $SUDO_PROMPT (can be overridden by `-p')
+     */
+    if ((prompt = getenv("SUDO_PROMPT")) == NULL)
+	prompt = PASSPROMPT;
+
     /*
      * parse our arguments
      */
     sudo_mode = parse_args();
-
-    switch(sudo_mode) {
-	case MODE_VERSION :
-	case MODE_HELP :
+ 
+    switch (sudo_mode) {
+	case MODE_VERSION:
+	case MODE_HELP:
 	    (void) printf("CU Sudo version %s\n", version);
 	    if (sudo_mode == MODE_VERSION)
 		exit(0);
 	    else
 		usage(0);
 	    break;
-	case MODE_VALIDATE :
+	case MODE_VALIDATE:
 	    cmnd = "validate";
 	    break;
-    	case MODE_KILL :
+    	case MODE_KILL:
 	    cmnd = "kill";
 	    break;
-	case MODE_LIST :
+	case MODE_LIST:
 	    cmnd = "list";
 	    printmatches = 1;
 	    break;
@@ -247,7 +263,7 @@ int main(argc, argv)
     if ((sudo_mode & MODE_SHELL)) {
 	char **dst, **src = NewArgv;
 
-	NewArgv = (char **) malloc (sizeof(char *) * (++NewArgc));
+	NewArgv = (char **) malloc (sizeof(char *) * (++NewArgc + 1));
 	if (NewArgv == NULL) {
 	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
@@ -264,25 +280,25 @@ int main(argc, argv)
 	}
 
 	/* copy the args from Argv */
-	dst = NewArgv + 1;
-	*dst = (char *) NULL;
-	while (*src) {
-	    *dst++ = *src++;
-	}
+	for (dst = NewArgv + 1; (*dst = *src) != NULL; ++src, ++dst)
+	    ;
     }
 
     rtn = check_sudoers();	/* check mode/owner on _PATH_SUDO_SUDOERS */
     if (rtn != ALL_SYSTEMS_GO) {
 	log_error(rtn);
-	set_perms(PERM_FULL_USER);
+	set_perms(PERM_FULL_USER, sudo_mode);
 	inform_user(rtn);
 	exit(1);
     }
 
 #ifdef SECURE_PATH
     /* replace the PATH envariable with a secure one */
-    if (!user_is_exempt())
-	sudo_setenv("PATH", SECURE_PATH);
+    if (!user_is_exempt() && sudo_setenv("PATH", SECURE_PATH)) {
+	perror("malloc");
+	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	exit(1);
+    }
 #endif /* SECURE_PATH */
 
     if ((sudo_mode & MODE_RUN)) {
@@ -290,29 +306,34 @@ int main(argc, argv)
     } else if (sudo_mode == MODE_KILL) {
 	remove_timestamp();	/* remove the timestamp ticket file */
 	exit(0);
-    } else if (sudo_mode == MODE_LIST) {
-#ifndef NO_PASSWD
-	check_user();
-#endif /* NO_PASSWD */
-       	log_error(ALL_SYSTEMS_GO);
-	(void) validate(FALSE);
-	exit(0);
     }
 
-    add_env();			/* add in SUDO_* envariables */
+    add_env(!(sudo_mode & MODE_SHELL));	/* add in SUDO_* envariables */
 
     /* validate the user but don't search for "validate" */
-    rtn = validate((sudo_mode != MODE_VALIDATE));
+    rtn = validate((sudo_mode != MODE_VALIDATE && sudo_mode != MODE_LIST));
+
     switch (rtn) {
 
 	case VALIDATE_OK:
-#ifndef NO_PASSWD
-	    check_user();
-#endif /* NO_PASSWD */
+	case VALIDATE_OK_NOPASS:
+	    if (rtn != VALIDATE_OK_NOPASS) 
+		check_user();
 	    log_error(ALL_SYSTEMS_GO);
 	    if (sudo_mode == MODE_VALIDATE)
 		exit(0);
-	    set_perms(PERM_FULL_ROOT);
+	    else if (sudo_mode == MODE_LIST) {
+		list_matches();
+		exit(0);
+	    }
+
+	    /* become specified user or root */
+	    set_perms(PERM_RUNAS, sudo_mode);
+
+	    /* set $HOME for `sudo -H' */
+	    if ((sudo_mode & MODE_RESET_HOME) && runas_homedir)
+		(void) sudo_setenv("HOME", runas_homedir);
+
 #ifndef PROFILING
 	    if ((sudo_mode & MODE_BACKGROUND) && fork() > 0) {
 		exit(0);
@@ -339,22 +360,22 @@ int main(argc, argv)
 			exit(1);
 		    }
 		}
-
 		EXEC(cmnd, NewArgv);	/* run the command */
 	    }
 #else
 	    exit(0);
 #endif /* PROFILING */
-	    perror(cmnd);		/* exec failed! */
+	    /*
+	     * If we got here then the exec() failed...
+	     */
+	    (void) fprintf(stderr, "%s: ", Argv[0]);
+	    perror(cmnd);
 	    exit(-1);
 	    break;
 
-	case VALIDATE_NO_USER:
-	case VALIDATE_NOT_OK:
-	case VALIDATE_ERROR:
 	default:
 	    log_error(rtn);
-	    set_perms(PERM_FULL_USER);
+	    set_perms(PERM_FULL_USER, sudo_mode);
 	    inform_user(rtn);
 	    exit(1);
 	    break;
@@ -375,7 +396,6 @@ static void load_globals(sudo_mode)
     int sudo_mode;
 {
     char *p;
-    struct passwd *pw_ent;
 #ifdef FQDN
     struct hostent *h_ent;
 #endif /* FQDN */
@@ -385,17 +405,14 @@ static void load_globals(sudo_mode)
      * if necesary.  It is assumed that euid is 0 at this point so we
      * can read the shadow passwd file if necesary.
      */
-    user_pw_ent = sudo_getpwuid(getuid());
-    set_perms(PERM_ROOT);
-    set_perms(PERM_USER);
-    if (user_pw_ent == NULL) {
+    if ((user_pw_ent = sudo_getpwuid(getuid())) == NULL) {
 	/* need to make a fake user_pw_ent */
 	struct passwd pw_ent;
 	char pw_name[MAX_UID_T_LEN+1];
 
 	/* fill in uid and name fields with the uid */
 	pw_ent.pw_uid = getuid();
-	(void) sprintf(pw_name, "%ld", pw_ent.pw_uid);
+	(void) sprintf(pw_name, "%ld", (long) pw_ent.pw_uid);
 	pw_ent.pw_name = pw_name;
 	user_pw_ent = &pw_ent;
 
@@ -405,6 +422,10 @@ static void load_globals(sudo_mode)
 	exit(1);
     }
 
+    /* Set euid == user and ruid == root */
+    set_perms(PERM_ROOT, sudo_mode);
+    set_perms(PERM_USER, sudo_mode);
+
 #ifdef HAVE_TZSET
     (void) tzset();		/* set the timezone if applicable */
 #endif /* HAVE_TZSET */
@@ -412,16 +433,15 @@ static void load_globals(sudo_mode)
     /*
      * Need to get tty early since it's used for logging
      */
-    if ((tty = (char *) ttyname(0)) || (tty = (char *) ttyname(1))) {
-	if (strncmp(tty, _PATH_DEV, sizeof(_PATH_DEV) - 1) == 0)
-	    tty += sizeof(_PATH_DEV) - 1;
-	if ((tty = (char *) strdup(tty)) == NULL) {
+    if ((p = (char *) ttyname(0)) || (p = (char *) ttyname(1))) {
+	if (strncmp(p, _PATH_DEV, sizeof(_PATH_DEV) - 1) == 0)
+	    p += sizeof(_PATH_DEV) - 1;
+	if ((tty = (char *) strdup(p)) == NULL) {
 	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
-    } else
-	tty = "none";
+    }
 
 #ifdef UMASK
     (void) umask((mode_t)UMASK);
@@ -440,13 +460,13 @@ static void load_globals(sudo_mode)
      */
     if (!getwd(cwd)) {
 	/* try as root... */
-	set_perms(PERM_ROOT);
+	set_perms(PERM_ROOT, sudo_mode);
 	if (!getwd(cwd)) {
 	    (void) fprintf(stderr, "%s:  Can't get working directory!\n",
 			   Argv[0]);
 	    (void) strcpy(cwd, "unknown");
 	}
-	set_perms(PERM_USER);
+	set_perms(PERM_USER, sudo_mode);
     }
 
     /*
@@ -461,11 +481,10 @@ static void load_globals(sudo_mode)
     }
 #ifdef FQDN
     if ((h_ent = gethostbyname(host)) == NULL)
-       log_error(GLOBAL_HOST_UNREGISTERED);
+	log_error(GLOBAL_HOST_UNREGISTERED);
     else
-       strcpy(host, h_ent -> h_name);
-#endif /* FQDN
-    }
+	strcpy(host, h_ent -> h_name);
+#endif /* FQDN */
 
     /*
      * "host" is the (possibly fully-qualified) hostname and
@@ -482,7 +501,7 @@ static void load_globals(sudo_mode)
     } else {
 	shost = &host[0];
     }
-       
+
     /*
      * load a list of ip addresses and netmasks into
      * the interfaces array.
@@ -504,6 +523,9 @@ static int parse_args()
     int ret = MODE_RUN;			/* what mode is suod to be run in? */
     int excl = 0;			/* exclusive arg, no others allowed */
 
+    NewArgv = Argv + 1;
+    NewArgc = Argc - 1;
+
 #ifdef SHELL_IF_NO_ARGS
     if (Argc < 2) {			/* no options and no command */
 	ret |= MODE_SHELL;
@@ -514,8 +536,6 @@ static int parse_args()
 	usage(1);
 #endif /* SHELL_IF_NO_ARGS */
 
-    NewArgv = Argv + 1;
-    NewArgc = Argc - 1;
     while (NewArgc > 0 && NewArgv[0][0] == '-') {
 	if (NewArgv[0][1] != '\0' && NewArgv[0][2] != '\0') {
 	    (void) fprintf(stderr, "%s: Please use single character options\n",
@@ -534,6 +554,17 @@ static int parse_args()
 
 		prompt = NewArgv[1];
 
+		/* shift Argv over and adjust Argc */
+		NewArgc--;
+		NewArgv++;
+		break;
+	    case 'u':
+		/* must have an associated runas user */
+		if (NewArgv[1] == NULL)
+		    usage(1);
+
+		runas_user = NewArgv[1];
+
 		/* shift Argv over and adjust Argc */
 		NewArgc--;
 		NewArgv++;
@@ -563,6 +594,12 @@ static int parse_args()
 		break;
 	    case 's':
 		ret |= MODE_SHELL;
+#ifdef SHELL_SETS_HOME
+		ret |= MODE_RESET_HOME;
+#endif /* SHELL_SETS_HOME */
+		break;
+	    case 'H':
+		ret |= MODE_RESET_HOME;
 		break;
 	    case '-':
 		NewArgc--;
@@ -585,6 +622,10 @@ static int parse_args()
 	NewArgv++;
     }
 
+    if (NewArgc > 0 && (ret == MODE_VALIDATE || ret == MODE_KILL ||
+			ret == MODE_LIST))
+	usage(1);
+
     return(ret);
 }
 
@@ -600,7 +641,7 @@ static int parse_args()
 static void usage(exit_val)
     int exit_val;
 {
-    (void) fprintf(stderr, "usage: %s -V | -h | -l | -v | -k | [-b] [-p prompt] -s | <command>\n", Argv[0]);
+    (void) fprintf(stderr, "usage: %s -V | -h | -l | -v | -k | -H | [-b] [-p prompt] [-u username/#uid] -s | <command>\n", Argv[0]);
     exit(exit_val);
 }
 
@@ -613,16 +654,60 @@ static void usage(exit_val)
  *  this function adds sudo-specific variables into the environment
  */
 
-static void add_env()
+static void add_env(contiguous)
+    int contiguous;
 {
     char idstr[MAX_UID_T_LEN + 1];
+    size_t size;
+    char *buf;
 
-    /* add the SUDO_COMMAND envariable */
-    if (sudo_setenv("SUDO_COMMAND", cmnd)) {
+    /* add the SUDO_COMMAND envariable (cmnd + args) */
+    size = strlen(cmnd) + 1;
+    if (NewArgc > 1) {
+	char *to, **from;
+
+	if (contiguous) {
+	    size += (size_t) NewArgv[NewArgc-1] + strlen(NewArgv[NewArgc-1]) -
+		    (size_t) NewArgv[1] + 1;
+	} else {
+	    for (from = &NewArgv[1]; *from; from++)
+		size += strlen(*from) + 1;
+	}
+
+	if ((buf = (char *) malloc(size)) == NULL) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
+	}
+
+	/*
+	 * Copy the command and it's arguments info buf
+	 */
+	(void) strcpy(buf, cmnd);
+	to = buf + strlen(cmnd);
+	for (from = &NewArgv[1]; *from; from++) {
+	    *to++ = ' ';
+	    (void) strcpy(to, *from);
+	    to += strlen(*from);
+	}
+    } else {
+	buf = cmnd;
+    }
+    if (sudo_setenv("SUDO_COMMAND", buf)) {
 	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
+    if (NewArgc > 1)
+	(void) free(buf);
+
+    /* grab a pointer to the flat arg string from the environment */
+    if (NewArgc > 1 && (cmnd_args = getenv("SUDO_COMMAND"))) {
+	if ((cmnd_args = strchr(cmnd_args, ' ')))
+	    cmnd_args++;
+	else
+	    cmnd_args = NULL;
+    }
 
     /* add the SUDO_USER envariable */
     if (sudo_setenv("SUDO_USER", user_name)) {
@@ -640,12 +725,20 @@ static void add_env()
     }
 
     /* add the SUDO_GID envariable */
-    (void) sprintf(idstr, "%ld", (long) getegid());
+    (void) sprintf(idstr, "%ld", (long) user_gid);
     if (sudo_setenv("SUDO_GID", idstr)) {
 	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
+
+    /* set PS1 if SUDO_PS1 is set */
+    if ((buf = getenv("SUDO_PS1")))
+	if (sudo_setenv("PS1", buf)) {
+	    perror("malloc");
+	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	    exit(1);
+	}
 }
 
 
@@ -654,7 +747,7 @@ static void add_env()
  *
  *  load_cmnd()
  *
- *  This function sets the cmnd global variable based on Argv
+ *  This function sets the cmnd global variable
  */
 
 static void load_cmnd(sudo_mode)
@@ -683,38 +776,71 @@ static void load_cmnd(sudo_mode)
  *
  *  check_sudoers()
  *
- *  This function check to see that the sudoers file is readable,
- *  owned by SUDOERS_OWNER, and not writable by anyone else.
+ *  This function check to see that the sudoers file is owned by
+ *  uid SUDOERS_UID, gid SUDOERS_GID and is mode SUDOERS_MODE.
  */
 
 static int check_sudoers()
 {
     struct stat statbuf;
-    struct passwd *pw_ent;
-    int fd;
+    int fd = -1, rootstat;
     char c;
     int rtn = ALL_SYSTEMS_GO;
 
-    if (!(pw_ent = getpwnam(SUDOERS_OWNER)))
-	return(SUDOERS_NO_OWNER);
+    /*
+     * Fix the mode and group on sudoers file from old default.
+     * Only works if filesystem is readable/writable by root.
+     */
+    set_perms(PERM_ROOT, 0);
+    if ((rootstat = lstat(_PATH_SUDO_SUDOERS, &statbuf)) == 0 &&
+	SUDOERS_UID == statbuf.st_uid && SUDOERS_MODE != 0400 &&
+	(statbuf.st_mode & 0007777) == 0400) {
 
-    set_perms(PERM_SUDOERS);
+	if (chmod(_PATH_SUDO_SUDOERS, SUDOERS_MODE) == 0) {
+	    (void) fprintf(stderr, "%s: fixed mode on %s\n",
+		Argv[0], _PATH_SUDO_SUDOERS);
+	    if (statbuf.st_gid != SUDOERS_GID) {
+		if (!chown(_PATH_SUDO_SUDOERS,GID_NO_CHANGE,SUDOERS_GID)) {
+		    (void) fprintf(stderr, "%s: set group on %s\n",
+			Argv[0], _PATH_SUDO_SUDOERS);
+		    statbuf.st_gid = SUDOERS_GID;
+		} else {
+		    (void) fprintf(stderr,"%s: Unable to set group on %s: ",
+			Argv[0], _PATH_SUDO_SUDOERS);
+		    perror("");
+		}
+	    }
+	} else {
+	    (void) fprintf(stderr, "%s: Unable to fix mode on %s: ",
+		Argv[0], _PATH_SUDO_SUDOERS);
+	    perror("");
+	}
+    }
 
-    if ((fd = open(_PATH_SUDO_SUDOERS, O_RDONLY)) < 0 || read(fd, &c, 1) == -1)
-	rtn = NO_SUDOERS_FILE;
-    else if (lstat(_PATH_SUDO_SUDOERS, &statbuf))
+    /*
+     * Sanity checks on sudoers file.  Must be done as sudoers
+     * file owner.  We already did a stat as root, so use that
+     * data if we can't stat as sudoers file owner.
+     */
+    set_perms(PERM_SUDOERS, 0);
+
+    if (lstat(_PATH_SUDO_SUDOERS, &statbuf) != 0 && rootstat != 0)
 	rtn = NO_SUDOERS_FILE;
     else if (!S_ISREG(statbuf.st_mode))
 	rtn = SUDOERS_NOT_FILE;
-    else if (statbuf.st_uid != pw_ent -> pw_uid)
+    else if ((statbuf.st_mode & 0007777) != SUDOERS_MODE)
+	rtn = SUDOERS_WRONG_MODE;
+    else if (statbuf.st_uid != SUDOERS_UID || statbuf.st_gid != SUDOERS_GID)
 	rtn = SUDOERS_WRONG_OWNER;
-    else if ((statbuf.st_mode & 0000066))
-	rtn = SUDOERS_RW_OTHER;
+    else if ((fd = open(_PATH_SUDO_SUDOERS, O_RDONLY)) == -1 ||
+	     read(fd, &c, 1) == -1)
+	rtn = NO_SUDOERS_FILE;
 
-    (void) close(fd);
+    if (fd != -1)
+	(void) close(fd);
 
-    set_perms(PERM_ROOT);
-    set_perms(PERM_USER);
+    set_perms(PERM_ROOT, 0);
+    set_perms(PERM_USER, 0);
 
     return(rtn);
 }
@@ -728,67 +854,115 @@ static int check_sudoers()
  *  this function sets real and effective uids and gids based on perm.
  */
 
-void set_perms(perm)
+void set_perms(perm, sudo_mode)
     int perm;
+    int sudo_mode;
 {
     struct passwd *pw_ent;
 
-    switch(perm) {
-	case        PERM_ROOT :
+    switch (perm) {
+	case PERM_ROOT:
 				if (setuid(0)) {
 				    perror("setuid(0)");
 				    exit(1);
 				}
 			      	break;
 
-	case   PERM_FULL_ROOT :
-				if (setuid(0)) {  
-				    perror("setuid(0)");
-				    exit(1);
-				}
+	case PERM_USER: 
+    	    	    	        (void) setgid(user_gid);
 
-				if (!(pw_ent = getpwuid(0))) {
-				    perror("getpwuid(0)");
-				} else if (setgid(pw_ent->pw_gid)) {
-				    perror("setgid");
-    	    	    	    	}
-			      	break;
-
-	case        PERM_USER : 
     	    	    	        if (seteuid(user_uid)) {
     	    	    	            perror("seteuid(user_uid)");
     	    	    	            exit(1); 
     	    	    	        }
 			      	break;
 				
-	case   PERM_FULL_USER : 
+	case PERM_FULL_USER: 
 				if (setuid(0)) {
 				    perror("setuid(0)");
 				    exit(1);
 				}
 
+    	    	    	        (void) setgid(user_gid);
+
 				if (setuid(user_uid)) {
-				    perror("setuid(uid)");
+				    perror("setuid(user_uid)");
 				    exit(1);
 				}
 
 			      	break;
+	case PERM_RUNAS:
+				if (setuid(0)) {
+				    perror("setuid(0)");
+				    exit(1);
+				}
+				
+				/* XXX - add group/gid support */
+				if (*runas_user == '#') {
+				    if (setuid(atoi(runas_user + 1))) {
+					(void) fprintf(stderr,
+					    "%s: cannot set uid to %s: ",
+					    Argv[0], runas_user);
+					perror("");
+					exit(1);
+				    }
+				} else {
+				    if (!(pw_ent = getpwnam(runas_user))) {
+					(void) fprintf(stderr,
+					    "%s: no passwd entry for %s!\n",
+					    Argv[0], runas_user);
+					exit(1);
+				    }
 
-	case   PERM_SUDOERS : 
+				    if (setgid(pw_ent->pw_gid)) {
+					(void) fprintf(stderr,
+					    "%s: cannot set gid to %d: ",  
+					    Argv[0], pw_ent->pw_gid);
+					perror("");
+					exit(1);
+				    }
+
+				    if (setuid(pw_ent->pw_uid)) {
+					(void) fprintf(stderr,
+					    "%s: cannot set uid to %d: ",  
+					    Argv[0], pw_ent->pw_uid);
+					perror("");
+					exit(1);
+				    }
+				    if (sudo_mode & MODE_RESET_HOME)
+					runas_homedir = pw_ent->pw_dir;
+				}
+
+				break;
+	case PERM_SUDOERS: 
 				if (setuid(0)) {
 				    perror("setuid(0)");
 				    exit(1);
 				}
 
-				if (!(pw_ent = getpwnam(SUDOERS_OWNER))) {
-				    (void) fprintf(stderr, "%s: no passwd entry for sudoers file owner (%s)\n", Argv[0], SUDOERS_OWNER);
+				if (setgid(SUDOERS_GID)) {
+				    perror("setgid(SUDOERS_GID)");
 				    exit(1);
-				} else if (seteuid(pw_ent->pw_uid)) {
-				    (void) fprintf(stderr, "%s: ",
-							   SUDOERS_OWNER);
-				    perror("");
-				    exit(1);
-    	    	    	    	}
+				}
+
+				/*
+				 * If SUDOERS_UID == 0 and SUDOERS_MODE
+				 * is group readable we use a non-zero
+				 * uid in order to avoid NFS lossage.
+				 * Using uid 1 is a bit bogus but should
+				 * work on all OS's.
+				 */
+				if (SUDOERS_UID == 0) {
+				    if ((SUDOERS_MODE & 040) && seteuid(1)) {
+					perror("seteuid(1)");
+					exit(1);
+				    }
+				} else {
+				    if (seteuid(SUDOERS_UID)) {
+					perror("seteuid(SUDOERS_UID)");
+					exit(1);
+				    }
+				}
 
 			      	break;
     }
