@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.4
+ *  CU sudo version 1.5.4
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Please send bugs, changes, problems to sudo-bugs@cs.colorado.edu
+ *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
  *
  *******************************************************************
  *
@@ -27,7 +27,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: tgetpass.c,v 1.37 1996/04/02 21:42:59 millert Exp $";
+static char rcsid[] = "$Id: tgetpass.c,v 1.52 1998/01/13 04:48:42 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -48,9 +48,9 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.37 1996/04/02 21:42:59 millert Exp $"
 #ifdef HAVE_SYS_BSDTYPES_H
 #include <sys/bsdtypes.h>
 #endif /* HAVE_SYS_BSDTYPES_H */
-#ifdef _AIX
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
-#endif /* _AIX */
+#endif /* HAVE_SYS_SELECT_H */
 #include <sys/time.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -64,6 +64,14 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.37 1996/04/02 21:42:59 millert Exp $"
 #include <sys/ioctl.h>
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
+#if (SHADOW_TYPE == SPW_SECUREWARE)
+#  ifdef __hpux
+#    include <hpsecurity.h>
+#  else
+#    include <sys/security.h>
+#  endif /* __hpux */ 
+#  include <prot.h>
+#endif /* SPW_SECUREWARE */
 
 #include <pathnames.h>
 #include "compat.h"
@@ -82,20 +90,19 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.37 1996/04/02 21:42:59 millert Exp $"
  *  and input will time out based on the value of timeout.
  */
 
-char * tgetpass(prompt, timeout)
+char * tgetpass(prompt, timeout, user, host)
     const char *prompt;
     int timeout;
+    char *user;
+    char *host;
 {
 #ifdef HAVE_TERMIOS_H
     struct termios term;
-    tcflag_t svflagval;
 #else
 #ifdef HAVE_TERMIO_H
     struct termio term;
-    unsigned short svflagval;
 #else
     struct sgttyb ttyb;
-    int svflagval;
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 #ifdef POSIX_SIGNALS
@@ -104,16 +111,18 @@ char * tgetpass(prompt, timeout)
 #else
     int oldmask;
 #endif /* POSIX_SIGNALS */
-    int input, output, n;
+    int n, echo;
+    FILE *input, *output;
     static char buf[_PASSWD_LEN + 1];
     fd_set readfds;
     struct timeval tv;
+    char *p;
 
     /*
      * mask out SIGINT and SIGTSTP, should probably just catch and deal.
      */
 #ifdef POSIX_SIGNALS
-    (void) memset((VOID *)&mask, 0, sizeof(mask));
+    (void) sigemptyset(&mask);
     (void) sigaddset(&mask, SIGINT);
     (void) sigaddset(&mask, SIGTSTP);
     (void) sigprocmask(SIG_BLOCK, &mask, &oldmask);
@@ -125,9 +134,10 @@ char * tgetpass(prompt, timeout)
      * open /dev/tty for reading/writing if possible or use
      * stdin and stderr instead.
      */
-    if ((input = open(_PATH_TTY, O_RDWR)) < 0) {
-	input = fileno(stdin);
-	output = fileno(stderr);
+    if ((input = fopen(_PATH_TTY, "r+")) == NULL) {
+	input = stdin;
+	output = stderr;
+	(void) fflush(output);
     } else {
 	output = input;
     }
@@ -136,67 +146,113 @@ char * tgetpass(prompt, timeout)
      * turn off echo
      */
 #ifdef HAVE_TERMIOS_H
-    (void) tcgetattr(input, &term);
-    svflagval = term.c_lflag;
-    term.c_lflag &= ~ECHO;
-    (void) tcsetattr(input, TCSAFLUSH|TCSASOFT, &term);
+    (void) tcgetattr(fileno(input), &term);
+    if ((echo = (term.c_lflag & ECHO))) {
+	term.c_lflag &= ~ECHO;
+	(void) tcsetattr(fileno(input), TCSAFLUSH|TCSASOFT, &term);
+    }
 #else
 #ifdef HAVE_TERMIO_H
-    (void) ioctl(input, TCGETA, &term);
-    svflagval = term.c_lflag;
-    term.c_lflag &= ~ECHO;
-    (void) ioctl(input, TCSETA, &term);
+    (void) ioctl(fileno(input), TCGETA, &term);
+    if ((echo = (term.c_lflag & ECHO))) {
+	term.c_lflag &= ~ECHO;
+	(void) ioctl(fileno(input), TCSETA, &term);
+    }
 #else
-    (void) ioctl(input, TIOCGETP, &ttyb);
-    svflagval = ttyb.sg_flags;
-    ttyb.sg_flags &= ~ECHO;
-    (void) ioctl(input, TIOCSETP, &ttyb);
+    (void) ioctl(fileno(input), TIOCGETP, &ttyb);
+    if ((echo = (ttyb.sg_flags & ECHO))) {
+	ttyb.sg_flags &= ~ECHO;
+	(void) ioctl(fileno(input), TIOCSETP, &ttyb);
+    }
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
     /* print the prompt */
-    (void) write(output, prompt, strlen(prompt));
+    if (prompt) {
+	p = (char *) prompt;
+	do {
+	    /* expand %u -> username, %h -> host */
+	    switch (*p) {
+		case '%':   if (user && *(p+1) == 'u') {
+				(void) fputs(user, output);
+				p++;
+				break;
+			    } else if (host && *(p+1) == 'h') {
+				(void) fputs(host, output);
+				p++;
+				break;
+			    }
 
-    /* setup for select(2) */
-    FD_ZERO(&readfds);
-    FD_SET(input, &readfds);
+		default:    (void) fputc(*p, output);
+	    }
+	} while (*(++p));
+    }
 
-    /* set timeout for select */
-    tv.tv_sec = timeout;
-    tv.tv_usec = 0;
-
-    /* how many file descriptors may we have? */
-#ifdef HAVE_SYSCONF
-    n = sysconf(_SC_OPEN_MAX);
-#else
-    n = getdtablesize();
-#endif /* HAVE_SYSCONF */
+    /* rewind if necesary */
+    if (input == output) {
+	(void) fflush(output);
+	(void) rewind(output);
+    }
 
     /*
-     * get password or return empty string if nothing to read by timeout
+     * Timeout of <= 0 means no timeout
      */
-    buf[0] = '\0';
-    if (select(n, &readfds, NULL, NULL, &tv) > 0 &&
-	(n = read(input, buf, sizeof(buf))) > 0) {
-	buf[n - 1] = '\0';
+    if (timeout > 0) {
+	/* setup for select(2) */
+	FD_ZERO(&readfds);
+	FD_SET(fileno(input), &readfds);
+
+	/* set timeout for select */
+	tv.tv_sec = timeout;
+	tv.tv_usec = 0;
+
+	/*
+	 * get password or return empty string if nothing to read by timeout
+	 */
+	buf[0] = '\0';
+	if (select(fileno(input) + 1, &readfds, 0, 0, &tv) > 0 &&
+	    fgets(buf, sizeof(buf), input)) {
+	    n = strlen(buf);
+	    if (buf[n - 1] == '\n')
+		buf[n - 1] = '\0';
+	}
+    } else {
+	buf[0] = '\0';
+	if (fgets(buf, sizeof(buf), input)) {
+	    n = strlen(buf);
+	    if (buf[n - 1] == '\n')
+		buf[n - 1] = '\0';
+	}
     }
 
      /* turn on echo */
 #ifdef HAVE_TERMIOS_H
-    term.c_lflag = svflagval;
-    (void) tcsetattr(input, TCSAFLUSH|TCSASOFT, &term);
+    if (echo) {
+	term.c_lflag |= ECHO;
+	(void) tcsetattr(fileno(input), TCSAFLUSH|TCSASOFT, &term);
+    }
 #else
 #ifdef HAVE_TERMIO_H
-    term.c_lflag = svflagval;
-    (void) ioctl(input, TCSETA, &term);
+    if (echo) {
+	term.c_lflag |= ECHO;
+	(void) ioctl(fileno(input), TCSETA, &term);
+    }
 #else
-    ttyb.sg_flags = svflagval;
-    (void) ioctl(input, TIOCSETP, &ttyb);
+    if (echo) {
+	ttyb.sg_flags |= ECHO;
+	(void) ioctl(fileno(input), TIOCSETP, &ttyb);
+    }
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
+    /* rewind if necesary */
+    if (input == output) {
+	(void) fflush(output);
+	(void) rewind(output);
+    }
+
     /* print a newline since echo is turned off */
-    (void) write(output, "\n", 1);
+    (void) fputc('\n', output);
 
     /* restore old signal mask */
 #ifdef POSIX_SIGNALS
@@ -206,8 +262,8 @@ char * tgetpass(prompt, timeout)
 #endif
 
     /* close /dev/tty if that's what we opened */
-    if (input != fileno(stdin))
-	(void) close(input);
+    if (input != stdin)
+	(void) fclose(input);
 
     return(buf);
 }
