@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.2
+ *  CU sudo version 1.5.4
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: visudo.c,v 1.76 1996/10/07 05:06:41 millert Exp $";
+static char rcsid[] = "$Id: visudo.c,v 1.82 1998/01/13 04:48:42 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -128,7 +128,7 @@ int main(argc, argv)
     int argc;
     char **argv;
 {
-    char buf[BUFSIZ];			/* buffer used for copying files */
+    char buf[MAXPATHLEN*2];		/* buffer used for copying files */
     char * Editor = EDITOR;		/* editor to use (default is EDITOR */
     int sudoers_fd;			/* sudoers file descriptor */
     int stmp_fd;			/* stmp file descriptor */
@@ -156,7 +156,12 @@ int main(argc, argv)
 	usage();
 
     /* user_pw_ent needs to point to something... */
-    user_pw_ent = getpwuid(getuid());
+    if ((user_pw_ent = getpwuid(getuid())) == NULL) {
+	(void) fprintf(stderr, "%s: Can't find you in the passwd database: ",
+	    Argv[0]);
+	perror(stmp);
+	exit(1);
+    }
 
 #ifdef ENV_EDITOR
     /*
@@ -180,30 +185,32 @@ int main(argc, argv)
 	}
 	(void) fprintf(stderr, "%s: ", Argv[0]);
 	perror(stmp);
-	Exit(1);
+	Exit(-1);
     }
 
     /* install signal handler to clean up stmp */
     setup_signals();
 
     sudoers_fd = open(sudoers, O_RDONLY);
-    if (sudoers_fd < 0) {
+    if (sudoers_fd < 0 && errno != ENOENT) {
 	(void) fprintf(stderr, "%s: ", Argv[0]);
 	perror(sudoers);
-	Exit(1);
+	Exit(-1);
     }
 
     /*
      * Copy the data
      */
-    while ((n = read(sudoers_fd, buf, sizeof(buf))) > 0)
-	if (write(stmp_fd, buf, n) != n) {
-	    (void) fprintf(stderr, "%s: Write failed: ", Argv[0]);
-	    perror("");
-	    Exit(1);
-	}
+    if (sudoers_fd >= 0) {
+	while ((n = read(sudoers_fd, buf, sizeof(buf))) > 0)
+	    if (write(stmp_fd, buf, n) != n) {
+		(void) fprintf(stderr, "%s: Write failed: ", Argv[0]);
+		perror("");
+		Exit(-1);
+	    }
 
-    (void) close(sudoers_fd);
+	(void) close(sudoers_fd);
+    }
     (void) close(stmp_fd);
 
     /*
@@ -213,6 +220,11 @@ int main(argc, argv)
 	/*
 	 * Build up a buffer to execute
 	 */
+	if (strlen(Editor) + strlen(stmp) + 30 > sizeof(buf)) {
+	    (void) fprintf(stderr, "%s: Buffer too short (line %d).\n",
+			   __LINE__, Argv[0]);
+	    Exit(-1);
+	}
 	if (parse_error == TRUE)
 	    (void) sprintf(buf, "%s +%d %s", Editor, errorlineno, stmp);
 	else
@@ -228,7 +240,7 @@ int main(argc, argv)
 		(void) fprintf(stderr,
 		    "%s: Can't stat temporary file (%s), %s unchanged.\n",
 		    Argv[0], stmp, sudoers);
-		Exit(1);
+		Exit(-1);
 	    }
 
 	    /* check for zero length file */
@@ -236,7 +248,7 @@ int main(argc, argv)
 		(void) fprintf(stderr,
 		    "%s: Zero length temporary file (%s), %s unchanged.\n",
 		    Argv[0], stmp, sudoers);
-		Exit(1);
+		Exit(-1);
 	    }
 
 	    /*
@@ -252,23 +264,23 @@ int main(argc, argv)
 		(void) fprintf(stderr,
 		    "%s: Can't re-open temporary file (%s), %s unchanged.\n",
 		    Argv[0], stmp, sudoers);
-		Exit(1);
+		Exit(-1);
 	    }
 
 	    /* clean slate for each parse */
 	    init_parser();
 
 	    /* parse the sudoers file */
-	    if (yyparse()) {
+	    if (yyparse() && parse_error != TRUE) {
 		(void) fprintf(stderr,
-		    "%s: Failed to parse temporary file (%s), %s unchanged.\n",
+		    "%s: Failed to parse temporary file (%s), unknown error.\n",
 		    Argv[0], stmp, sudoers);
-		Exit(1);
+		parse_error = TRUE;
 	    }
 	} else {
 	    (void) fprintf(stderr, "%s: Editor (%s) failed, %s unchanged.\n",
 		Argv[0], Editor, sudoers);
-	    Exit(1);
+	    Exit(-1);
 	}
 
 	/*
@@ -293,14 +305,14 @@ int main(argc, argv)
 	    "%s: Unable to set (uid, gid) of %s to (%d, %d): ",
 	    Argv[0], stmp, SUDOERS_UID, SUDOERS_GID);
 	perror("");
-	Exit(1);
+	Exit(-1);
     }
     if (chmod(stmp, SUDOERS_MODE)) {
 	(void) fprintf(stderr,
 	    "%s: Unable to change mode of %s to %o: ",
 	    Argv[0], stmp, SUDOERS_MODE);
 	perror("");
-	Exit(1);
+	Exit(-1);
     }
 
     /*
@@ -324,7 +336,7 @@ int main(argc, argv)
 			      "%s: Cannot alocate memory, %s unchanged: ",
 			      Argv[0], sudoers);
 		perror("");
-		Exit(1);
+		Exit(-1);
 	    }
 
 	    /* Build up command and execute it */
@@ -333,14 +345,14 @@ int main(argc, argv)
 		(void) fprintf(stderr,
 			       "%s: Command failed: '%s', %s unchanged.\n",
 			       Argv[0], tmpbuf, sudoers);
-		Exit(1);
+		Exit(-1);
 	    }
 	    (void) free(tmpbuf);
 	} else {
 	    (void) fprintf(stderr, "%s: Error renaming %s, %s unchanged: ",
 				   Argv[0], stmp, sudoers);
 	    perror("");
-	    Exit(1);
+	    Exit(-1);
 	}
 
     return(0);
@@ -394,7 +406,7 @@ int netgr_matches(n, h, u)
 static void usage()
 {
     (void) fprintf(stderr, "usage: %s [-V]\n", Argv[0]);
-    Exit(1);
+    Exit(-1);
 }
 
 
@@ -404,13 +416,18 @@ static void usage()
  *
  *  Unlinks the sudoers temp file (if it exists) and exits.
  *  Used in place of a normal exit() and as a signal handler.
+ *  A positive parameter is considered to be a signal and is reported.
  */
 
 static RETSIGTYPE Exit(sig)
     int sig;
 {
     (void) unlink(stmp);
-    exit(sig);
+
+    if (sig > 0)
+	(void) fprintf(stderr, "%s exiting, caught signal %d.\n", Argv[0], sig);
+
+    exit(-sig);
 }
 
 
@@ -429,16 +446,12 @@ static char whatnow()
 
     do {
 	ok = FALSE;
-	(void) printf("What now? ");
-	if ((choice = fgetc(stdin)) != '\n')
-	    while (fgetc(stdin) != '\n')
+	(void) fputs("What now? ", stdout);
+	if ((choice = getchar()) != '\n')
+	    while (getchar() != '\n')
 		;
 
-	/* safely force to lower case */
-	if (isupper(choice))
-	    choice = tolower(choice);
-
-	if (choice == 'e' || choice == 'x' || choice == 'q')
+	if (choice == 'e' || choice == 'x' || choice == 'Q')
 	    ok = TRUE;
 
 	/* help message if they gavce us garbage */
@@ -463,7 +476,7 @@ static void whatnow_help()
     (void) printf("Options are:\n");
     (void) printf("  (e)dit sudoers file again\n");
     (void) printf("  e(x)it without saving changes to sudoers file\n");
-    (void) printf("  (q)uit and save changes to sudoers file (DANGER!)\n\n");
+    (void) printf("  (Q)uit and save changes to sudoers file (DANGER!)\n\n");
 }
 
 
